package fri.util.javascript;

import java.io.Reader;

import javax.script.Bindings;

import net.sourceforge.htmlunit.corejs.javascript.Context;
import net.sourceforge.htmlunit.corejs.javascript.Scriptable;
import net.sourceforge.htmlunit.corejs.javascript.ScriptableObject;

import com.gargoylesoftware.htmlunit.BrowserVersion;

import fri.util.javascript.bom.ConsoleMock;
import fri.util.javascript.htmlunit.PrintlnLogger;

/**
 * A JavaScript Object in Rhino seem to have no hasOwnProperty() function!
 * 
 * @author Fritz Ritzberger Oct 2, 2014
 */
public class JavaScriptRhinoRunnerWithHtmlUnit extends JavaScriptRunnerWithHtmlUnit
{
	public static void main(String [] args) throws Exception {
		new ArgumentRunner(args).run(
			new ArgumentRunner.JavaScriptRunnerFactory() {
				public JavaScriptRunner newJavaScriptRunner(String webPage, BrowserVersion browserVersion)	{
					return new JavaScriptRhinoRunnerWithHtmlUnit(webPage, browserVersion);
				}
			}
		);
	}
	

    private Context context;
	
	public JavaScriptRhinoRunnerWithHtmlUnit(String webPage, BrowserVersion browserVersion) {
		super(webPage, browserVersion);
		this.context = Context.enter();
	}
	
	/** @return a Rhino Scope. */
	@Override
	protected Object createScriptEngine() {
        final Scriptable scope = context.initStandardObjects(getWindow());
        
        if (BrowserVersion.getDefault().equals(browserVersion))	{
        	ScriptableObject.putProperty(scope, "console", Context.javaToJS(new ConsoleMock(), scope));	// else: Console has no such function "log"
        	// for some reason only default (IE-8) permits to set a console
        }
        else	{	// configure loggers
        	webClient.getWebConsole().setLogger(new PrintlnLogger());
        }
        
		return scope;
	}

	/** Overridden to do nothing because bindings are done in createScriptEngine(). */
	protected void addGlobalBindings(Bindings bindings) {
	}
	
	/** Receiving (and casting) a Rhino Scope instead of a script engine. */
	@Override
	protected void execute(Object scriptEngine, Reader reader, String origin) throws Exception {
		assert context != null;
		
		final Scriptable scope = (Scriptable) scriptEngine;
		context.evaluateReader(scope, reader, origin, 1, null);
	}

	protected void cleanUp() {
		super.cleanUp();
		
		Context.exit();
		context = null;
	}

}
