"use strict";

/**
 * @author Fritz Ritzberger Sep 24, 2014
 */

var assertProperty = function(value, name)	{
	if (typeof value === "undefined")
		console.log("ERROR: "+name+", no such property");
	else
		console.log("SUCCESS: "+name+", value = "+value);
}

var assertFunction = function(name, context, functionToCall, parameters)	{
	if ( ! functionToCall )	{
		console.log("ERROR: "+name+", no such function");
	}
	else	{
		try	{
			var returnValue = functionToCall.apply(context, parameters);
			console.log("SUCCESS: "+name+", return = "+returnValue);
		}
		catch (error)	{
			console.log("ERROR: "+name+", '"+error+"'");
		}
	}
}

var assertFunctionExists = function(name, context, functionToCall, parameters)	{
	if ( ! functionToCall )	{
		console.log("ERROR: "+name+", no such function");
	}
	else	{
		console.log("SUCCESS: "+name+", function exists (call test skipped)");
	}
}

assertProperty(window, "window");
assertProperty(document, "document");
assertProperty(history, "history");
assertProperty(location, "location");
assertProperty(navigator, "navigator");
assertProperty(screen, "screen");
assertProperty(XMLHttpRequest, "XMLHttpRequest");

assertProperty(window.closed, "window.closed");
assertProperty(window.defaultStatus, "window.defaultStatus");
assertProperty(window.document, "window.document");
assertProperty(window.frames, "window.frames");
assertProperty(window.history, "window.history");
assertProperty(window.innerHeight, "window.innerHeight");
assertProperty(window.innerWidth, "window.innerWidth");
assertProperty(window.length, "window.length");
assertProperty(window.location, "window.location");
assertProperty(window.name, "window.name");
assertProperty(window.navigator, "window.navigator");
assertProperty(window.opener, "window.opener");
assertProperty(window.outerHeight, "window.outerHeight");
assertProperty(window.outerWidth, "window.outerWidth");
assertProperty(window.pageXOffset, "window.pageXOffset");
assertProperty(window.pageYOffset, "window.pageYOffset");
assertProperty(window.parent, "window.parent");
assertProperty(window.screen, "window.screen");
assertProperty(window.screenLeft, "window.screenLeft");
assertProperty(window.screenTop, "window.screenTop");
assertProperty(window.screenX, "window.screenX");
assertProperty(window.screenY, "window.screenY");
assertProperty(window.self, "window.self");
assertProperty(window.status, "window.status");
assertProperty(window.top, "window.top");
assertProperty(window.URL, "window.URL");
assertFunction("window.alert()", window, window.alert, [ "alert" ]);
assertFunction("window.blur()", window, window.blur);
assertFunction("window.confirm()", window, window.confirm, [ "confirm" ]);
assertFunction("window.createPopup()", window, window.createPopup);
assertFunction("window.focus()", window, window.focus);
assertFunction("window.moveBy()", window, window.moveBy, [ 0, 0 ]);
assertFunction("window.moveTo()", window, window.moveTo, [ 0, 0 ]);
assertFunction("window.open()", window, window.open, [ "http://www.example.com", "example", "", false ]);	// HtmlUnit needs 4 params
assertFunction("window.close()", window, window.close);
assertFunction("window.print()", window, window.print);
assertFunction("window.prompt()", window, window.prompt, [ "prompt" ]);
assertFunction("window.resizeBy()", window, window.resizeBy, [ 0, 0 ]);
assertFunction("window.resizeTo()", window, window.resizeTo, [ 0, 0 ]);
assertFunction("window.scrollBy()", window, window.scrollBy, [ 0, 0 ]);
assertFunction("window.scrollTo()", window, window.scrollTo, [ 0, 0 ]);
assertFunction("window.setInterval()", window, window.setInterval, [ function(){}, 1000, "JavaScript" ]);
assertFunction("window.clearInterval()", window, window.clearInterval, [ 0 ]);
assertFunction("window.setTimeout()", window, window.setTimeout, [ function(){}, 1000, "JavaScript" ]);
assertFunction("window.clearTimeout()", window, window.clearTimeout, [ 0 ]);
assertFunction("window.stop()", window, window.stop);

assertProperty(document.all, "document.all");
assertFunction("document.addEventListener()", document, document.addEventListener, [ "eventType", function(event){}, false ]);
assertProperty(document.activeElement, "document.activeElement");
assertFunction("document.adoptNode()", document, document.adoptNode, [ document.createElement("p") ])
assertProperty(document.anchors, "document.anchors");
assertProperty(document.applets, "document.applets");
assertProperty(document.baseURI, "document.baseURI");
assertProperty(document.body, "document.body");
assertProperty(document.cookie, "document.cookie");
assertFunction("document.cloneNode()", document, document.cloneNode, [ document.createElement("p") ]);
assertFunction("document.createAttribute()", document, document.createAttribute, [ "testAttribute" ]);
assertFunction("document.createComment()", document, document.createComment, [ "testComment" ]);
assertFunction("document.createDocumentFragment()", document, document.createDocumentFragment);
assertFunction("document.createElement()", document, document.createElement, [ "div" ]);
assertFunction("document.createTextNode()", document, document.createTextNode, [ "testTextNode" ]);
assertProperty(document.doctype, "document.doctype");
assertProperty(document.documentElement, "document.documentElement");
assertProperty(document.documentURI, "document.documentURI");
assertProperty(document.domain, "document.domain");
assertProperty(document.embeds, "document.embeds");
assertProperty(document.forms, "document.forms");
assertFunction("document.getElementById()", document, document.getElementById, [ "testId" ]);
assertFunction("document.getElementsByClassName()", document, document.getElementsByClassName, [ "testClassname" ]);
assertFunction("document.getElementsByName()", document, document.getElementsByName, [ "div" ]);
assertFunction("document.getElementsByTagName()", document, document.getElementsByTagName, [ "div" ]);
assertProperty(document.head, "document.head");
assertProperty(document.images, "document.images");
assertProperty(document.implementation, "document.implementation");
assertFunction("document.importNode()", document, document.importNode, [ document.createElement("div"), false ]);
assertProperty(document.inputEncoding, "document.inputEncoding");
assertProperty(document.lastModified, "document.lastModified");
assertProperty(document.links, "document.links");
assertFunction("document.normalize()", document, document.normalize);
assertFunction("document.normalizeDocument()", document, document.normalizeDocument);
assertFunction("document.open(2)", document, document.open, [ "http://www.example.com", "replace" ]);	// w3schools wants 2 params
assertFunction("document.open(4)", document, document.open, [ "http://www.example.com", "example", "", false ]);	// HtmlUnit also takes 4 params
assertFunction("document.close()", document, document.close);
assertFunction("document.write()", document, document.write, [ "Hello World" ]);
assertFunction("document.writeln()", document, document.writeln, [ "Hello Worldln" ]);
assertFunction("document.close()", document, document.close);
assertFunction("document.querySelector()", document, document.querySelector, [ "cssSelector" ]);
assertFunction("document.querySelectorAll()", document, document.querySelectorAll, [ "cssSelector" ]);
assertProperty(document.readyState, "document.readyState");
assertProperty(document.referrer, "document.referrer");
assertFunction("document.removeEventListener()", document, document.removeEventListener, [ "eventType", function(event){}, false ]);
assertProperty(document.scripts, "document.scripts");
assertProperty(document.title, "document.title");
assertProperty(document.URL, "document.URL");

var element = document.createElement("div");
var child = document.createElement("p");
assertFunction("node.appendChild()", element, element.appendChild, [ child ]);
assertFunction("node.addEventListener()", element, element.addEventListener, [ "eventType", function(event){}, false ]);
assertProperty(element.childNodes, "element.childNodes");
assertProperty(element.clientHeight, "element.clientHeight");
assertProperty(element.clientWidth, "element.clientWidth");
assertProperty(element.clientTop, "element.clientTop");
assertProperty(element.clientLeft, "element.clientLeft");
assertFunction("node.cloneNode(true)", element, element.cloneNode, [ true ]);
assertFunction("element.focus()", element, element.focus);
assertFunction("element.getAttribute()", element, element.getAttribute, [ "testAttributeName" ]);
assertFunction("element.getElementsByClassName()", element, element.getElementsByClassName, [ "cssClassName" ]);
assertFunction("element.getElementsByTagName()", element, element.getElementsByTagName, [ "p" ]);
assertFunction("node.getUserData()", element, element.getUserData);
assertFunction("element.hasAttribute()", element, element.hasAttribute, [ "testAttributeName" ]);
assertFunction("node.hasChildNodes()", element, element.hasChildNodes);
assertFunction("element.insertAdjacentHTML()", element, element.insertAdjacentHTML, [ "afterend", "<p>Hi</p>" ]);
assertFunction("node.insertBefore()", element, element.insertBefore, [ document.createElement("span"), child ]);
assertProperty(element.innerHTML, "element.innerHTML");
assertProperty(element.nextSibling, "node.nextSibling");
assertProperty(element.nodeName, "node.nodeName");
assertFunction("node.normalize()", element, element.normalize);
assertProperty(element.offsetHeight, "element.offsetHeight");
assertProperty(element.offsetLeft, "element.offsetLeft");
assertProperty(element.offsetTop, "element.offsetTop");
assertProperty(element.offsetWidth, "element.offsetWidth");
assertProperty(element.outerHTML, "element.outerHTML");
assertProperty(element.ownerDocument, "node.ownerDocument");
assertProperty(element.previousSibling, "node.previousSibling");
assertProperty(element.parentNode, "node.parentNode");
assertProperty(element.parentElement, "node.parentElement");
assertFunction("element.querySelector()", element, element.querySelector, [ "cssSelector" ]);
assertFunction("element.querySelectorAll()", element, element.querySelectorAll, [ "cssSelector" ]);
assertProperty(element.readyState, "element.readyState");
assertFunction("element.removeAttribute()", element, element.removeAttribute, [ "testAttributeName" ]);
var child2 = document.createElement("p");
assertFunction("node.replaceChild()", element, element.replaceChild, [ child2, child ]);
assertFunction("node.removeChild()", element, element.removeChild, [ child2 ]);
assertFunction("node.removeEventListener()", element, element.removeEventListener, [ "eventType", function(event){}, false ]);
assertProperty(element.style, "element.style");
assertFunction("element.setAttribute()", element, element.setAttribute, [ "testAttributeName", "testAttributeValue" ]);
assertFunction("node.setUserData()", element, element.setUserData, [ "userData" ]);
assertProperty(element.scrollHeight, "element.scrollHeight");
assertProperty(element.scrollLeft, "element.scrollLeft");
assertProperty(element.scrollTop, "element.scrollTop");
assertProperty(element.scrollWidth, "element.scrollWidth");
assertProperty(element.tagName, "element.tagName");
assertProperty(element.textContent, "node.textContent");

var attribute = document.createAttribute("testAttribute");
assertProperty(attribute.name, "attribute.name");
assertProperty(attribute.value, "attribute.value");
assertProperty(attribute.isId, "attribute.isId");

assertProperty(history.length, "history.length");
assertFunction("history.back()", history, history.back);
assertFunction("history.forward()", history, history.forward);
assertFunction("history.go(string)", history, history.go, [ "http://www.example.com" ]);
assertFunction("history.go(int)", history, history.go, [ 0 ]);

assertProperty(location.hash, "location.hash");
assertProperty(location.host, "location.host");
assertProperty(location.hostname, "location.hostname");
assertProperty(location.href, "location.href");
assertProperty(location.origin, "location.origin");
assertProperty(location.pathname, "location.pathname");
assertProperty(location.port, "location.port");
assertProperty(location.protocol, "location.protocol");
assertProperty(location.search, "location.search");
assertFunctionExists("location.reload()", location, location.reload, [ false ]);	// HtmlUnit crashes with NPE
assertFunctionExists("location.assign()", location, location.assign, [ "http://www.example.com" ]);	// HtmlUnit crashes with NPE
assertFunctionExists("location.replace()", location, location.replace, [ "http://www.example.com" ]);	// HtmlUnit crashes with NPE

assertProperty(navigator.appCodeName, "navigator.appCodeName");
assertProperty(navigator.appName, "navigator.appName");
assertProperty(navigator.appVersion, "navigator.appVersion");
assertProperty(navigator.buildID, "navigator.buildID");
assertProperty(navigator.cookieEnabled, "navigator.cookieEnabled");
assertProperty(navigator.language, "navigator.language");
assertProperty(navigator.onLine, "navigator.onLine");
assertProperty(navigator.platform, "navigator.platform");
assertProperty(navigator.product, "navigator.product");
assertProperty(navigator.systemLanguage, "navigator.systemLanguage");
assertProperty(navigator.userAgent, "navigator.userAgent");

assertProperty(screen.availHeight, "screen.availHeight");
assertProperty(screen.availWidth, "screen.availWidth");
assertProperty(screen.colorDepth, "screen.colorDepth");
assertProperty(screen.height, "screen.height");
assertProperty(screen.pixelDepth, "screen.pixelDepth");
assertProperty(screen.width, "screen.width");

var ajax = new XMLHttpRequest();
ajax.onload = function() {};
ajax.timeout = 10000;
assertFunction("ajax.abort()", ajax, ajax.abort);
assertFunction("ajax.open()", ajax, ajax.open, [ "GET", "http://www.this-page-intentionally-left-blank.org/", false ]);
assertFunction("ajax.overrideMimeType()", ajax, ajax.overrideMimeType, [ "text/xml" ]);
assertFunction("ajax.setRequestHeader()", ajax, ajax.setRequestHeader, [ "my", "value" ]);
assertFunction("ajax.send()", ajax, ajax.send);
assertProperty(ajax.onload, "ajax.onload");
assertProperty(ajax.readyState, "ajax.readyState");
assertProperty(ajax.response, "ajax.response");
assertProperty(ajax.responseText, "ajax.responseText");
assertProperty(ajax.responseType, "ajax.responseType");
assertProperty(ajax.responseXML, "ajax.responseXML");
assertProperty(ajax.status, "ajax.status");
assertProperty(ajax.statusText, "ajax.statusText");
assertProperty(ajax.timeout, "ajax.timeout");
assertProperty(ajax.upload, "ajax.upload");
assertFunction("ajax.getAllResponseHeaders()", ajax, ajax.getAllResponseHeaders);
assertFunction("ajax.getResponseHeader()", ajax, ajax.getResponseHeader, [ "Server" ]);
